/*
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	Copyright (C) 2010 John Crispin <blogic@openwrt.org>
 */

#ifndef _OWSIP_BACKEND_H__
#define _OWSIP_BACKEND_H__

#include <stdio.h>

#include <libubox/list.h>
#include <pjsua-lib/pjsua.h>
#include <pjsua-lib/pjsua_internal.h>

struct owsip;
struct call;

struct backend {
	struct list_head list;

	int ports;	// how many ports are there
	char *name;	// the name of the backend

	void (*init)(struct owsip *owsip);	// gets called when the app starts up (before sip init)
	void (*exit)(struct owsip *owsip);	// gets called when the app cleans up

	void (*mainloop)(struct owsip *owsip);	// a backend can ship its own mainloop

	void (*start)(struct call *c); // gets called when a new call arrives
	void (*stop)(struct call *c); // gets called when a call ends

	void (*connect)(struct call *c);	// connect a calls RTP
	void (*disconnect)(struct call *c);	// disconnect a calls RTP

	void (*media_config)(pjsua_media_config *media_cfg);	// allow backend to override mediacfg settings 
};

void backend_register(struct backend *b);
struct backend* backend_get(const char *name);
char* backend_get_default(void);

#define BACKEND(name) \
	void  __attribute__ ((constructor)) \
	backend##name##start(void) \
	{ \
		printf("registering backend %s\n", #name); \
		backend_register(&name); \
	}
#endif
