/*
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	Copyright (C) 2010 John Crispin <blogic@openwrt.org>
 */

#ifndef _OWSIP_CALL_H__
#define _OWSIP_CALL_H__

#include <libubox/list.h>

#include "../backend/backend.h"

struct call;
struct call_handler
{
	struct list_head list;

	char *name;	// the name of the call_handler
	void (*start)(struct call *c);	// the call handler can start taking control of the call
	void (*connect)(struct call *c);	// the call should start the rtp stream
};

struct call
{
	pjsua_call_id call_id;	// pjsip call id
	pj_timer_entry timer;	// the timer used to trigger call handler (if one is set)
	struct pjmedia_transport *transport;	// the pj transport layer
	struct tapi_stream *stream;	// the rtp stream
	struct rule *rule;	// the rule the matched the call in the dispatcher
	struct call_handler *handler;	// the call handler set by the rule
	void *handler_priv;	// handlers priv data
	struct backend *backend;	// the backend handling us
	void *backend_priv;	// the backends priv data
};

enum {
	CALL_START,	// new call started
	CALL_STOP,	// call ended
	CALL_CONNECT,	// call was picked up and the backend should connect the rtp stream
};

void call_state(struct owsip *owsip, pjsua_call_id call_id, int state);

void call_handler_register(struct call_handler *c);

#define CALL(name) \
	void  __attribute__ ((constructor)) \
	call##name##start(void) \
	{ \
		printf("registering call handler %s\n", #name); \
		call_handler_register(&name); \
	}
#endif
