/*
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	Copyright (C) 2010 John Crispin <blogic@openwrt.org>
 */

#include "detector.h"

#define TOUT_DIAL		3000
#define TOUT_IMPULSE	200

static const pj_time_val tdial = {
	.sec = 0,
	.msec = TOUT_DIAL,
};

static const pj_time_val timpulse = {
	.sec = 0,
	.msec = TOUT_IMPULSE,
};

static void
detector_timeout(pj_timer_heap_t *timer_heap, struct pj_timer_entry *entry)
{
	OWSIP_TRACER("enter");

	struct detector *d = (struct detector*) entry->user_data;
	d->nr[d->nr_cnt] = '\0';
	d->state = DETECTOR_CALLING;
	if(d->number)
		d->number(d, d->nr);
}

static void
detector_impulse(pj_timer_heap_t *timer_heap, struct pj_timer_entry *entry)
{
	OWSIP_TRACER("enter");

	//struct detector *d = (struct detector*) entry->user_data;
}

void
detector_init(struct detector *d)
{
	OWSIP_TRACER("enter");

	d->timeout.cb = detector_timeout;
	d->timeout.user_data = d;
	d->impulse.cb = detector_impulse;
	d->impulse.user_data = d;
	d->state = DETECTOR_UNKNOWN;
	d->nr_cnt = 0;
}

void
detector_feed(struct detector *d, enum detector_event e, char data)
{
	OWSIP_TRACER("enter");

	switch(e)
	{
	case DETECTOR_HOOK:
		if(!data)
		{
			d->state = DETECTOR_NONE;
			pjsua_cancel_timer(&d->timeout);
			pjsua_cancel_timer(&d->impulse);
			d->nr_cnt = 0;
		} else {
			d->state = DETECTOR_DIALING;
		}
		if(d->event)
			d->event(d, DETECTOR_HOOK, data);
		break;
	case DETECTOR_DIGIT:
		if(d->state == DETECTOR_DIALING || d->state == DETECTOR_UNKNOWN)
		{
			if(d->nr_cnt < MAX_DIGIT)
				d->nr[d->nr_cnt++] = data;
			pjsua_cancel_timer(&d->timeout);
			pjsua_schedule_timer(&d->timeout, &tdial);
		}else if(d->event && (d->state == DETECTOR_CALLING))
			d->event(d, DETECTOR_DIGIT, data);
		break;
	}
}
